import { EC2Client } from '@aws-sdk/client-ec2'
import { getInstanceAndAmiDetails } from '.'
import { BuildStep, VMSpec } from '../../Driver'

type ImageNameGenerationData = {
  buildInstanceType: string
  region: string
  sourceAmiId: string
  sshUsername: string
  buildSteps: BuildStep[]
}

const buildInstanceTypesByCpuArchitecture = {
  x64: 't2.xlarge',
  arm64: 't4g.xlarge',
}

export async function getImageNameGenerationData(vmSpec: VMSpec): Promise<ImageNameGenerationData> {
  const region = await new EC2Client().config.region()

  const { instanceDetail, amiDetail } = getInstanceAndAmiDetails(vmSpec)
  const buildInstanceType = buildInstanceTypesByCpuArchitecture[instanceDetail.cpuArchitecture]
  const sourceAmiId = amiDetail.ids[region]
  const sshUsername = amiDetail.sshUsername

  return {
    buildInstanceType,
    region,
    sourceAmiId,
    sshUsername,
    buildSteps: vmSpec.build_steps ?? [],
  }
}
